# THIS FILE IS GENERATED FROM ORANGE-WIDGET-BASE SETUP.PY
short_version = '4.17.0'
version = '4.17.0'
full_version = '4.17.0'
git_revision = 'd829100d1401fc116251f52b17c2590927beed3e'
release = True

if not release:
    version = full_version
    short_version += ".dev"
